#
#  Copyright 1999-2003 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

package provide MxGui 0.66

proc mxgui_create_motor_widget { motor_record } {
	set motor_name [ $motor_record cget -name ]

	set motor_widget .motor_widget_
	append motor_widget $motor_name

# Setup the window and the title bar

	set status [ catch { toplevel $motor_widget -class MxMotor } result ]

# If the window already exists, bring it to the top.

	if { $status != 0 } {
		raise $motor_widget
		focus $motor_widget
		return
	}

# Hide the window for now

	wm withdraw $motor_widget

	wm title $motor_widget "Motor '$motor_name'"

#####################################################################
#        Display the motor name at the top of the window            #
#####################################################################

	frame $motor_widget.name_frame
	pack $motor_widget.name_frame -expand yes -fill x

	label $motor_widget.name_frame.name -text $motor_name \
				-background $MxGui::textbackground
	pack $motor_widget.name_frame.name

#####################################################################
#                 Create the main control window                    #
#####################################################################

# Create a separator

	frame $motor_widget.sep -height 2 -borderwidth 1 -relief sunken
	pack $motor_widget.sep -expand yes -fill x

# Create the main window frame

	set main $motor_widget.main

	frame $main
	pack $main

# Create the high limit line

	label $main.highlimit_label -text "High limit"

	set text [ format "%g" [ $motor_record get_field positive_limit ] ]

	label $main.highlimit_value -foreground blue -text $text

	frame $main.highlimit_hit
	canvas $main.highlimit_hit.on -height 18 -width 18

	$main.highlimit_hit.on create oval \
			2 2 14 14 -fill red -outline red -width 0

	grid $main.highlimit_label $main.highlimit_value $main.highlimit_hit \
			-padx 10 -pady 5 -sticky w

# Create the position line

	label $main.position_label -text "Position"

	set text [ format "%g" [ $motor_record get_position ] ]

	label $main.position_value -foreground blue -text $text

	label $main.moving -foreground limegreen -width 10

	grid $main.position_label $main.position_value $main.moving \
			-padx 10 -pady 5 -sticky w

# Create the destination line

	label $main.destination_label -text "Destination"

	iwidgets::entryfield $main.destination_value

	$main.destination_value insert 0 [ $motor_record get_field position ]

#	$main.destination_value configure -textbackground cadetblue1
	$main.destination_value configure -validate \
					{ iwidgets::Entryfield::real %P }
	$main.destination_value configure -command \
	    [ itcl::code mxgui_motor_move_absolute $motor_widget $motor_record ]

	label $main.units -text [ $motor_record get_field units ]

	grid $main.destination_label $main.destination_value $main.units \
			-padx 10 -pady 5 -sticky w

# Create the low limit line

	label $main.lowlimit_label -text "Low limit"

	set text [ format "%g" [ $motor_record get_field negative_limit ] ]

	label $main.lowlimit_value -foreground blue -text $text

	frame $main.lowlimit_hit
	canvas $main.lowlimit_hit.on -height 18 -width 18

	$main.lowlimit_hit.on create oval \
			2 2 14 14 -fill red -outline red -width 0

	grid $main.lowlimit_label $main.lowlimit_value $main.lowlimit_hit \
			-padx 10 -pady 5 -sticky w

#####################################################################
#          Create the tweak box and stop button window              #
#####################################################################

# Create a separator

	frame $motor_widget.sep2 -height 2 -borderwidth 1 -relief sunken
	pack $motor_widget.sep2 -expand yes -fill x -pady 5

# Create the bottom window

	set bottom $motor_widget.bottom

	frame $bottom
	pack $bottom

# Create the tweak window frame

	frame $bottom.tweak

# Create the label

	label $bottom.tweak.label -text "Tweak"
	pack $bottom.tweak.label -side left

# Create the down arrow button

	button $bottom.tweak.down -text "<" -padx 2 -pady 0 \
    -command [ itcl::code mxgui_motor_tweak_down $motor_widget $motor_record ]

#	  -background cadetblue1 -activebackground lightcyan \
#    -command [ itcl::code mxgui_motor_tweak_down $motor_widget $motor_record ]

	pack $bottom.tweak.down -side left

# Create the tweak distance field

	iwidgets::entryfield $bottom.tweak.distance -width 10

	$bottom.tweak.distance insert 0 [ $motor_record cget -tweak_distance ]

#	$bottom.tweak.distance configure -textbackground cadetblue1
	$bottom.tweak.distance configure -validate \
					{ iwidgets::Entryfield::real %P }
	$bottom.tweak.distance configure -command \
	    [ itcl::code mxgui_motor_tweak_set $motor_widget $motor_record ]

	pack $bottom.tweak.distance -side left

# Create the up arrow button

	button $bottom.tweak.up -text ">" -padx 2 -pady 0 \
    -command [ itcl::code mxgui_motor_tweak_up $motor_widget $motor_record ]

#	  -background cadetblue1 -activebackground lightcyan \
#    -command [ itcl::code mxgui_motor_tweak_up $motor_widget $motor_record ]

	pack $bottom.tweak.up -side left

# Add an abort button to the right side of this line.

	label $bottom.blankspace -width 5

	button $bottom.abort -text "Abort Move" -background red \
    -activebackground pink -command [ itcl::code $motor_record soft_abort ]

# Put the buttons in a grid.

	grid $bottom.tweak $bottom.blankspace $bottom.abort 

#####################################################################
# Add a dismiss button to allow the user to get rid of this window. #
#####################################################################

	frame $motor_widget.sep3 -height 2 -borderwidth 1 -relief sunken
	pack $motor_widget.sep3 -expand yes -fill x -pady 5

	button $motor_widget.dismiss -text Dismiss \
		-command [ itcl::code destroy $motor_widget ]

	pack $motor_widget.dismiss

	frame $motor_widget.sep4 -height 2 -borderwidth 1
	pack $motor_widget.sep4 -expand yes -fill x -pady 2

# Make the window appear on the screen.

	wm deiconify $motor_widget

#####################################################################
#     Arrange for some of the fields to be periodically updated.    #
#####################################################################

	after 1 [ list mxgui_motor_widget_update $motor_widget $motor_record ]
}

#
# This procedure updates some of the fields in the motor widget periodically.
#

proc mxgui_motor_widget_update { motor_widget motor_record } {
	set main $motor_widget.main

	set position [ format "%g" [ $motor_record get_position ] ]

# Update the position value

	if [catch { $main.position_value configure -text $position }] {
		return
	}

	update

# Update the motion status

	set moving [ $motor_record is_busy ]

	if { $moving != 0 } {
		set newtext "Moving"
	} else {
		set newtext ""
	}

	if [catch { $main.moving configure -text $newtext }] {
		return
	}

	update

# Update the limit status

	set limit_hit [ $motor_record positive_limit_hit ]

	if { $limit_hit != 0 } {
		set update_command [list pack]
	} else {
		set update_command [list pack forget]
	}

	if [catch { eval $update_command $main.highlimit_hit.on }] {
		return
	}

	update

	set limit_hit [ $motor_record negative_limit_hit ]

	if { $limit_hit != 0 } {
		set update_command [list pack]
	} else {
		set update_command [list pack forget]
	}

	if [catch { eval $update_command $main.lowlimit_hit.on }] {
		return
	}

	update

# Reschedule this procedure to be called again.

	after $MxGui::motor_polling_interval \
		[ list mxgui_motor_widget_update $motor_widget $motor_record ]
}

#
# This procedure starts a motor move when the return key is pressed
# in the destination field.
#

proc mxgui_motor_move_absolute { motor_widget motor_record } {
	set new_position [ $motor_widget.main.destination_value get ]

	set motor_name [ $motor_record cget -name ]

	$motor_record move_absolute $new_position

	$motor_widget.main.moving configure -text "Moving"
}

#
# This procedure records the tweak distance in the motor object.
#

proc mxgui_motor_tweak_set { motor_widget motor_record } {
	set tweak_distance [ $motor_widget.bottom.tweak.distance get ]

	$motor_record configure -tweak_distance $tweak_distance
}

#
# The following two procedures implement the tweak buttons.
#

proc mxgui_motor_tweak_up { motor_widget motor_record } {
	set tweak_distance [ $motor_record cget -tweak_distance ]

	set old_position [ $motor_record get_position ]

	set new_position [ expr $old_position + $tweak_distance ]

	$motor_record move_absolute $new_position

	$motor_widget.main.moving configure -text "Moving"
}

proc mxgui_motor_tweak_down { motor_widget motor_record } {
	set tweak_distance [ $motor_record cget -tweak_distance ]

	set old_position [ $motor_record get_position ]

	set new_position [ expr $old_position - $tweak_distance ]

	$motor_record move_absolute $new_position

	$motor_widget.main.moving configure -text "Moving"
}

