#
#  Copyright 1999-2003 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

package provide MxGui 0.66

proc show_scalers_command { record_list } {

	# Create the show scalers window

	set mainwindow .show_scalers_window

	set status [ catch { toplevel $mainwindow } result ]

	# If the window already exists, bring it to the top.

	if { $status != 0 } {
		raise $mainwindow
		focus $mainwindow
		return
	}

	wm title $mainwindow "Scaler Control Window"

	label $mainwindow.status -fg red \
			-text "Not Yet Initialized"

	pack $mainwindow.status

	iwidgets::scrolledframe $mainwindow.scalerframe \
		-hscrollmode none -vscrollmode dynamic \
		-height 400 -width 300

	pack $mainwindow.scalerframe -expand yes -fill both

	set window [ $mainwindow.scalerframe childsite ]

	set row 0

	label $window.scaler_name -text "Scaler Name" -width 20
	label $window.value -text "Value" -width 12
	label $window.dark_current -text "Dark Current" -width 12
	label $window.subtraction_enabled \
			-text "Subtraction\nEnabled"

	grid $window.scaler_name $window.value $window.dark_current \
		$window.subtraction_enabled -row $row

	incr row

	set scalerlist \
	  [ $record_list get_multiple_records "device" "scaler" ".*" 0 ]

	set num_scalers [ llength $scalerlist ]

	# Don't make the field $mainwindow.num_scalers visible.  It is
	# just intended as a place to store the number of scalers.

	label $mainwindow.num_scalers -text $num_scalers

	for { set i 0 } { $i < $num_scalers } { incr i } {

		set thisscaler $window.scaler$i

		set scalername [ lindex $scalerlist $i ]

		set thisrecord [ $record_list get_record $scalername ]

		label ${thisscaler}_name -text $scalername \
				-anchor w \
				-font $MxGui::monospaced_font \
				-bg $MxGui::textbackground

		label ${thisscaler}_value \
				-anchor e \
				-font $MxGui::monospaced_font \
				-bg $MxGui::textbackground

		label ${thisscaler}_dark_current \
				-anchor e \
				-font $MxGui::monospaced_font \
				-bg $MxGui::textbackground

		label ${thisscaler}_subtraction_enabled \
				-anchor w \
				-font $MxGui::monospaced_font \
				-bg $MxGui::textbackground

		set scaler_flags [ $thisrecord get_field "scaler_flags" ]

		set subtraction_enabled [ expr ( $scaler_flags & 0x3 ) ]

		if { $subtraction_enabled } {
			${thisscaler}_subtraction_enabled configure -text Yes
		} else {
			${thisscaler}_subtraction_enabled configure -text No
		}

		grid ${thisscaler}_name ${thisscaler}_value \
			${thisscaler}_dark_current \
			${thisscaler}_subtraction_enabled \
			-sticky news -row $row

		incr row
	}

	#---------

	set bottom $mainwindow.bottom

	frame $bottom
	pack $bottom -fill x

	frame $bottom.sep1 -height 2 -borderwidth 1 -relief sunken
	pack $bottom.sep1 -expand yes -fill x

	frame $bottom.measurement_frame
	pack $bottom.measurement_frame

	set time_field $bottom.measurement_frame.time_field

	iwidgets::entryfield $time_field -labeltext "Measurement Time:"

	$time_field insert 0 $MxGui::measurement_time

	$time_field configure -validate \
				{ iwidgets::Entryfield::real %P }
	$time_field configure -command \
	    [ itcl::code update_measurement_time $time_field ]

	pack $time_field -side left

	label $bottom.measurement_frame.units -text Seconds
	pack $bottom.measurement_frame.units -side left

	#---------

	frame $bottom.sep2 -height 2 -borderwidth 1 -relief sunken
	pack $bottom.sep2 -expand yes -fill x

	# Do _not_ make $bottom.abort visible with pack or grid.

	label $bottom.abort -text 0

	button $bottom.abort_button -bg red \
		-activebackground pink \
		-text "Stop Measurement" \
		-command [ itcl::code scaler_stop_counting $bottom.abort ]

	pack $bottom.abort_button

	#---------

	frame $bottom.sep3 -height 2 -borderwidth 1 -relief sunken
	pack $bottom.sep3 -expand yes -fill x

	set buttonbox $bottom.buttonbox

	iwidgets::buttonbox $buttonbox

	$buttonbox add Count -text Count
	$buttonbox add MeasureDarkCurrent -text "Measure\nDark Current"
	$buttonbox add Dismiss -text Dismiss
	$buttonbox default Count

	$buttonbox buttonconfigure Count -command [ itcl::code \
		count_scalers $record_list $mainwindow ]

	$buttonbox buttonconfigure MeasureDarkCurrent -command [ itcl::code \
		measure_dark_current $record_list $mainwindow ]

	$buttonbox buttonconfigure Dismiss -command [ itcl::code \
		destroy_scaler_window $mainwindow ]

	pack $buttonbox

	update_show_scalers $record_list $mainwindow 1 OK
}

proc scaler_stop_counting { window } {

	$window configure -text 1

	update idletasks
}

proc update_measurement_time { time_field } {

	set MxGui::measurement_time [ $time_field get ]
}

proc update_show_scalers { record_list mainwindow measurement_time status } {

	if { $status == "Abort" } {

		$mainwindow.status configure -fg red
		$mainwindow.status configure \
			-text "Measurement Aborted"

		return
	}

	$mainwindow.status configure -fg yellow
	$mainwindow.status configure -text "Reading Scalers"

	update

	set window [ $mainwindow.scalerframe childsite ]

	set num_scalers [ $mainwindow.num_scalers cget -text ]

	for { set i 0 } { $i < $num_scalers } { incr i } {

		set thisscaler $window.scaler$i

		set scalername [ ${thisscaler}_name cget -text ]

		set thisrecord [ $record_list get_record $scalername ]

		set value [ $thisrecord read ]

		${thisscaler}_value configure -text \
			[ format "%d" $value ]

		${thisscaler}_dark_current configure -text \
			[ format "%.2f" [ $thisrecord get_dark_current ] ]
	}

	$mainwindow.status configure -fg green
	$mainwindow.status configure -text "Done"

	set framewidth [ winfo width $window ]

	$mainwindow.scalerframe configure -width $framewidth
}

proc destroy_scaler_window { mainwindow } {

	set time_field $mainwindow.bottom.measurement_frame.time_field

	set MxGui::measurement_time [ $time_field get ]

	destroy $mainwindow
}

#################

proc count_scalers { record_list mainwindow } {

	$mainwindow.bottom.abort configure -text 0

	set time_field $mainwindow.bottom.measurement_frame.time_field

	set measurement_time [ $time_field get ]

	set status [ catch \
		{ $record_list get_record "timer1" } timer_record ]

	if { $status != 0 } {
		set t "The required record 'timer1' was not found."

		Mx::error_message "No Timer Record" $t "" ""

		return
	}

	set status_text $mainwindow.status

	$status_text configure -text "Counting"
	$status_text configure -fg yellow

	update idletasks

	$timer_record start $measurement_time

	after 1 [ list count_scalers_callback $record_list \
			$timer_record $measurement_time $mainwindow ]
}

proc count_scalers_callback { record_list timer_record \
				measurement_time mainwindow } {

	set abort [ $mainwindow.bottom.abort cget -text ]

	if { $abort != 0 } {

		$mainwindow.bottom.abort configure -text 0

		$timer_record stop

		update_show_scalers $record_list \
			$mainwindow $measurement_time Abort

		return
	}

	if [ $timer_record is_busy ] {

		after 100 [ list count_scalers_callback $record_list \
				$timer_record $measurement_time $mainwindow ]
	} else {
		update_show_scalers $record_list \
			$mainwindow $measurement_time OK
	}
}

#################

proc measure_dark_current { record_list mainwindow } {

	$mainwindow.bottom.abort configure -text 0

	set time_field $mainwindow.bottom.measurement_frame.time_field

	set measurement_time [ $time_field get ]

	set status [ catch \
		{ $record_list get_record "timer1" } timer_record ]

	if { $status != 0 } {
		set t "The required record 'timer1' was not found."

		Mx::error_message "No Timer Record" $t "" ""

		return
	}

	# Setup a dialog with the user to get the number of measurements.

	set window .dark_current_setup_dialog

	set status [ catch { toplevel $window } result ]

	# If the window already exists, bring it to the top.

	if { $status != 0 } {
		raise $window
		focus $window
		return
	}

	wm withdraw $window
	wm title $window "Dark Current Measurement Setup"

	#-----

	set main $window.main

	frame $main
	pack $main -ipadx 5 -ipady 5

	label $main.time_label -text "Enter Measurement Time: "

	iwidgets::entryfield $main.measurement_time

	$main.measurement_time configure -validate \
			{ iwidgets::Entryfield::real %P }

	grid $main.time_label $main.measurement_time -sticky w 

	label $main.num_label -text "Enter Number of Measurements: "

	iwidgets::entryfield $main.num_measurements

	$main.num_measurements configure -validate \
			{ iwidgets::Entryfield::integer %P }

	grid $main.num_label $main.num_measurements -sticky w

	#-----

	frame $window.sep -height 2 -borderwidth 1 -relief sunken
	pack $window.sep -expand yes -fill x

	frame $window.b
	pack $window.b

	set buttonbox $window.b.buttonbox

	iwidgets::buttonbox $buttonbox -padx 0 -pady 2

	$buttonbox add Continue -text Continue -command \
		[ itcl::code perform_dark_current_measurement \
		  $record_list $timer_record $mainwindow $window ]

	$buttonbox add Cancel -text Cancel \
				-command [ itcl::code destroy $window ]

	$buttonbox default Continue

	pack $buttonbox

	wm deiconify $window
}

proc perform_dark_current_measurement { record_list timer_record \
					mainwindow setup_dialog } {

	$mainwindow.bottom.abort configure -text 0

	set num_measurements [ $setup_dialog.main.num_measurements get ]

	set measurement_time [ $setup_dialog.main.measurement_time get ]

	if { $num_measurements == "" } {
		Mx::error_message "No Number of Measurements" \
			"The number of measurements was not specified." "" ""

		return
	}
	if { $num_measurements <= 0 } {
		Mx::error_message "Illegal Number of Measurements" \
	"The number of measurements must be an integer greater than zero." "" ""

		return
	}
	if { $measurement_time == "" } {
		Mx::error_message "No Measurement Time" \
			"The measurement time was not specified." "" ""

		return
	}
	if { $measurement_time <= 0.0 } {
		Mx::error_message "Illegal Measurement Time" \
			"The measurement time must be greater than zero." "" ""

		return
	}

	destroy $setup_dialog

	set t "Measuring dark currents:  "
	append t \
	"$num_measurements measurements for $measurement_time seconds each."

	log_puts $t

	set window [ $mainwindow.scalerframe childsite ]

	set status_label $mainwindow.status

	set scaler_list \
	  [ $record_list get_multiple_records "device" "scaler" ".*" 0 ]

	set num_scalers [ llength $scaler_list ]

	for { set i 0 } { $i < $num_scalers } { incr i } {

		set scaler_name [ lindex $scaler_list $i ]

		set scaler_record_array($i) \
			[ $record_list get_record $scaler_name ]

		set scaler_record_sum($i) 0
	}

	for { set i 0 } { $i < $num_measurements } { incr i } {

		set n [ expr { $i + 1 } ]

		$status_label configure -fg orange
		$status_label configure -text \
			"Counting - Measurement $n of $num_measurements"

		update

		# Clear the scalers.

		for { set j 0 } { $j < $num_scalers } { incr j } {

			$scaler_record_array($j) clear
		}

		# Start the measurement.

		set exit_loop 0

		$timer_record start $measurement_time

		while { $exit_loop == 0 } {

			set abort [ $mainwindow.bottom.abort cget -text ]

			if { $abort != 0 } {

				$mainwindow.bottom.abort configure -text 0

				$timer_record stop

				update_show_scalers $record_list \
					$mainwindow $measurement_time Abort

				return
			}

			if { [ $timer_record is_busy ] == 0 } {
				break
			}
			after 10

			update
		}
		update_show_scalers $record_list \
			$mainwindow $measurement_time OK

		for { set j 0 } { $j < $num_scalers } { incr j } {

			set value [ $scaler_record_array($j) read_raw ]

			set old_sum $scaler_record_sum($j)

			set scaler_record_sum($j) [ expr {$old_sum + $value} ]
		}
	}

	for { set i 0 } { $i < $num_scalers } { incr i } {

		set dark_current \
			[ expr ( $scaler_record_sum($i) / $num_measurements ) ]

		set dark_current \
			[ expr ( $dark_current / $measurement_time ) ]

		$scaler_record_array($i) set_dark_current $dark_current
	}

	update_show_scalers $record_list $mainwindow $measurement_time OK
}

